@@ -54,12 +54,10 @@ struct stats_cpu *st_cpu[2];
 unsigned long long uptime[2]  = {0, 0};
 unsigned long long uptime0[2] = {0, 0};
 struct io_stats *st_iodev[2];
-struct io_nfs_stats *st_ionfs[2];
-struct io_hdr_stats *st_hdr_iodev, *st_hdr_ionfs;
+struct io_hdr_stats *st_hdr_iodev;
 struct io_dlist *st_dev_list;
 
 int iodev_nr = 0;	/* Nb of devices and partitions found */
-int ionfs_nr = 0;	/* Nb of NFS mounted directories found */
 int cpu_nr = 0;		/* Nb of processors on the machine */
 int dlist_idx = 0;	/* Nb of devices entered on the command line */
 int flags = 0;		/* Flag for common options and system state */
@@ -80,10 +78,15 @@ void usage(char *progname)
 {
 	fprintf(stderr, _("Usage: %s [ options ] [ <interval> [ <count> ] ]\n"),
 		progname);
-
+#ifdef DEBUG
 	fprintf(stderr, _("Options are:\n"
-			  "[ -c ] [ -d ] [ -N ] [ -n ] [ -h ] [ -k | -m ] [ -t ] [ -V ] [ -x ] [ -z ]\n"
+			  "[ -c ] [ -d ] [ -N ] [ -k | -m ] [ -t ] [ -V ] [ -x ] [ -z ]\n"
+			  "[ <device> [...] | ALL ] [ -p [ <device> [,...] | ALL ] ] [ --debuginfo ]\n"));
+#else
+	fprintf(stderr, _("Options are:\n"
+			  "[ -c ] [ -d ] [ -N ] [ -k | -m ] [ -t ] [ -V ] [ -x ] [ -z ]\n"
 			  "[ <device> [...] | ALL ] [ -p [ <device> [,...] | ALL ] ]\n"));
+#endif
 	exit(1);
 }
 
@@ -143,12 +146,11 @@ void init_stats(void)
 
 /*
  ***************************************************************************
- * Set every disk_io or nfs_io entry to inactive state (unregistered).
+ * Set every disk_io entry to inactive state (unregistered).
  *
  * IN:
- * @ioln_nr	Number of devices and partitions or NFS filesystems.
- * @st_hdr_ioln	Pointer on first structure describing a device/partition or
- * 		an NFS filesystem.
+ * @ioln_nr	Number of devices and partitions.
+ * @st_hdr_ioln	Pointer on first structure describing a device/partition.
  ***************************************************************************
  */
 void set_entries_inactive(int ioln_nr, struct io_hdr_stats *st_hdr_ioln)
@@ -166,9 +168,8 @@ void set_entries_inactive(int ioln_nr, struct io_hdr_stats *st_hdr_ioln)
  * Free inactive entries (mark them as unused).
  *
  * IN:
- * @ioln_nr	Number of devices and partitions or NFS filesystems.
- * @st_hdr_ioln	Pointer on first structure describing a device/partition or
- * 		an NFS filesystem.
+ * @ioln_nr	Number of devices and partitions.
+ * @st_hdr_ioln	Pointer on first structure describing a device/partition.
  ***************************************************************************
  */
 void free_inactive_entries(int ioln_nr, struct io_hdr_stats *st_hdr_ioln)
@@ -212,35 +213,6 @@ void salloc_device(int iodev_nr)
 	memset(st_hdr_iodev, 0, IO_HDR_STATS_SIZE * iodev_nr);
 }
 
-/*
- ***************************************************************************
- * Allocate and init I/O NFS directories structures.
- *
- * IN:
- * @ionfs_nr	Number of NFS filesystems.
- ***************************************************************************
- */
-void salloc_nfs(int ionfs_nr)
-{
-	int i;
-
-	for (i = 0; i < 2; i++) {
-		if ((st_ionfs[i] =
-		     (struct io_nfs_stats *) malloc(IO_NFS_STATS_SIZE * ionfs_nr)) == NULL) {
-			perror("malloc");
-			exit(4);
-		}
-		memset(st_ionfs[i], 0, IO_NFS_STATS_SIZE * ionfs_nr);
-	}
-
-	if ((st_hdr_ionfs =
-	     (struct io_hdr_stats *) malloc(IO_HDR_STATS_SIZE * ionfs_nr)) == NULL) {
-		perror("malloc");
-		exit(4);
-	}
-	memset(st_hdr_ionfs, 0, IO_HDR_STATS_SIZE * ionfs_nr);
-}
-
 /*
  ***************************************************************************
  * Allocate structures for devices entered on the command line.
@@ -346,16 +318,6 @@ void io_sys_init(void)
 	 * iodev_nr must be <> 0.
 	 */
 	salloc_device(iodev_nr);
-
-	/* Get number of NFS directories in /proc/self/mountstats */
-	if (DISPLAY_NFS(flags) &&
-	    ((ionfs_nr = get_nfs_mount_nr()) > 0)) {
-		flags |= I_F_HAS_NFS;
-		ionfs_nr += NR_NFS_PREALLOC;
-
-		/* Allocate structures for number of NFS directories found */
-		salloc_nfs(ionfs_nr);
-	}
 }
 
 /*
@@ -378,36 +340,26 @@ void io_sys_free(void)
 		if (st_iodev[i]) {
 			free(st_iodev[i]);
 		}
-		
-		/* Free I/O NFS directories structures */
-		if (st_ionfs[i]) {
-			free(st_ionfs[i]);
-		}
 	}
 	
 	if (st_hdr_iodev) {
 		free(st_hdr_iodev);
 	}
-	if (st_hdr_ionfs) {
-		free(st_hdr_ionfs);
-	}
 }
 
 /*
  ***************************************************************************
- * Save stats for current device, partition or NFS filesystem.
+ * Save stats for current device or partition.
  *
  * IN:
- * @name	Name of the device/partition or NFS filesystem.
+ * @name	Name of the device/partition.
  * @curr	Index in array for current sample statistics.
- * @st_io	Structure with device, partition or NFS statistics to save.
- * @ioln_nr	Number of devices and partitions or NFS filesystems.
- * @st_hdr_ioln	Pointer on structures describing a device/partition or an
- *		NFS filesystem.
+ * @st_io	Structure with device or partition to save.
+ * @ioln_nr	Number of devices and partitions.
+ * @st_hdr_ioln	Pointer on structures describing a device/partition.
  *
  * OUT:
- * @st_hdr_ioln	Pointer on structures describing a device/partition or an
- *		NFS filesystem.
+ * @st_hdr_ioln	Pointer on structures describing a device/partition.
  ***************************************************************************
  */
 void save_stats(char *name, int curr, void *st_io, int ioln_nr,
@@ -416,16 +368,15 @@ void save_stats(char *name, int curr, void *st_io, int ioln_nr,
 	int i;
 	struct io_hdr_stats *st_hdr_ioln_i;
 	struct io_stats *st_iodev_i;
-	struct io_nfs_stats *st_ionfs_i;
 
-	/* Look for device or NFS directory in data table */
+	/* Look for device in data table */
 	for (i = 0; i < ioln_nr; i++) {
 		st_hdr_ioln_i = st_hdr_ioln + i;
 		if (!strcmp(st_hdr_ioln_i->name, name)) {
 			break;
 		}
 	}
-	
+
 	if (i == ioln_nr) {
 		/*
 		 * This is a new device: look for an unused entry to store it.
@@ -437,14 +388,8 @@ void save_stats(char *name, int curr, void *st_io, int ioln_nr,
 				/* Unused entry found... */
 				st_hdr_ioln_i->used = TRUE; /* Indicate it is now used */
 				strcpy(st_hdr_ioln_i->name, name);
-				if (st_hdr_ioln == st_hdr_iodev) {
-					st_iodev_i = st_iodev[!curr] + i;
-					memset(st_iodev_i, 0, IO_STATS_SIZE);
-				}
-				else {
-					st_ionfs_i = st_ionfs[!curr] + i;
-					memset(st_ionfs_i, 0, IO_NFS_STATS_SIZE);
-				}
+				st_iodev_i = st_iodev[!curr] + i;
+				memset(st_iodev_i, 0, IO_STATS_SIZE);
 				break;
 			}
 		}
@@ -452,17 +397,11 @@ void save_stats(char *name, int curr, void *st_io, int ioln_nr,
 	if (i < ioln_nr) {
 		st_hdr_ioln_i = st_hdr_ioln + i;
 		st_hdr_ioln_i->active = TRUE;
-		if (st_hdr_ioln == st_hdr_iodev) {
-			st_iodev_i = st_iodev[curr] + i;
-			*st_iodev_i = *((struct io_stats *) st_io);
-		}
-		else {
-			st_ionfs_i = st_ionfs[curr] + i;
-			*st_ionfs_i = *((struct io_nfs_stats *) st_io);
-		}
+		st_iodev_i = st_iodev[curr] + i;
+		*st_iodev_i = *((struct io_stats *) st_io);
 	}
 	/*
-	 * else it was a new device or NFS directory
+	 * else it was a new device
 	 * but there was no free structure to store it.
 	 */
 }
@@ -758,112 +697,6 @@ void read_diskstats_stat(int curr)
 	free_inactive_entries(iodev_nr, st_hdr_iodev);
 }
 
-/*
- ***************************************************************************
- * Read NFS-mount directories stats from /proc/self/mountstats.
- *
- * IN:
- * @curr	Index in array for current sample statistics.
- ***************************************************************************
- */
-void read_nfs_stat(int curr)
-{
-	FILE *fp;
-	int sw = 0;
-	char line[8192];
-	char *xprt_line;
-	char nfs_name[MAX_NAME_LEN];
-	char mount[10], on[10], prefix[10], aux[32];
-	char operation[16];
-	struct io_nfs_stats snfs;
-	long int v1;
-
-	/* Every I/O NFS entry is potentially unregistered */
-	set_entries_inactive(ionfs_nr, st_hdr_ionfs);
-
-	if ((fp = fopen(NFSMOUNTSTATS, "r")) == NULL)
-		return;
-
-	sprintf(aux, "%%%ds %%10s %%10s",
-		MAX_NAME_LEN < 200 ? MAX_NAME_LEN : 200);
-
-	while (fgets(line, 256, fp) != NULL) {
-
-		/* read NFS directory name */
-		if (!strncmp(line, "device", 6)) {
-			sw = 0;
-			sscanf(line + 6, aux, nfs_name, mount, on);
-			if ((!strncmp(mount, "mounted", 7)) && (!strncmp(on, "on", 2))) {
-				sw = 1;
-			}
-		}
-
-		sscanf(line, "%10s", prefix);
-		if (sw && (!strncmp(prefix, "bytes:", 6))) {
-			/* Read the stats for the last NFS-mounted directory */
-			sscanf(strstr(line, "bytes:") + 6, "%llu %llu %llu %llu %llu %llu",
-			       &snfs.rd_normal_bytes, &snfs.wr_normal_bytes,
-			       &snfs.rd_direct_bytes, &snfs.wr_direct_bytes,
-			       &snfs.rd_server_bytes, &snfs.wr_server_bytes);
-			sw = 2;
-		}
-
-		if ((sw == 2) && (!strncmp(prefix, "xprt:", 5))) {
-			/*
-			 * Read extended statistic for the last NFS-mounted directory
-			 * - number of sent rpc requests.
-			 */
-			xprt_line = (strstr(line, "xprt:") + 6);
-			/* udp, tcp or rdma data */
-			if (!strncmp(xprt_line, "udp", 3)) {
-				/* port bind_count sends recvs (bad_xids req_u bklog_u) */
-				sscanf(strstr(xprt_line, "udp") + 4, "%*u %*u %lu",
-				       &snfs.rpc_sends);
-			}
-			if (!strncmp(xprt_line, "tcp", 3)) {
-				/*
-				 * port bind_counter connect_count connect_time idle_time
-				 * sends recvs (bad_xids req_u bklog_u)
-				 */
-				sscanf(strstr(xprt_line, "tcp") + 4,
-				       "%*u %*u %*u %*u %*d %lu",
-				       &snfs.rpc_sends);
-			}
-			if (!strncmp(xprt_line,"rdma", 4)) {
-				/*
-				 * 0(port) bind_count connect_count connect_time idle_time
-				 * sends recvs (bad_xids req_u bklog_u...)
-				 */
-				sscanf(strstr(xprt_line, "rdma") + 5,
-				       "%*u %*u %*u %*u %*d %lu",
-				       &snfs.rpc_sends);
-			}
-			sw = 3;
-		}
-
-		if ((sw == 3) && (!strncmp(prefix, "per-op", 6))) {
-			sw = 4;
-			while (sw == 4) {
-				fgets(line, 256, fp);
-				sscanf(line, "%15s %lu", operation, &v1);
-				if (!strncmp(operation, "READ:", 5)) {
-					snfs.nfs_rops = v1;
-				}
-				else if (!strncmp(operation, "WRITE:", 6)) {
-					snfs.nfs_wops = v1;
-					save_stats(nfs_name, curr, &snfs, ionfs_nr, st_hdr_ionfs);
-					sw = 0;
-				}
-			}
-		}
-	}
-
-	fclose(fp);
-
-	/* Free structures corresponding to unregistered devices */
-	free_inactive_entries(ionfs_nr, st_hdr_ionfs);
-}
-
 /*
  ***************************************************************************
  * Display CPU utilization.
@@ -938,35 +771,6 @@ void write_disk_stat_header(int *fctr)
 	}
 }
 
-/*
- ***************************************************************************
- * Display NFS stats header.
- *
- * OUT:
- * @fctr	Conversion factor.
- ***************************************************************************
- */
-void write_nfs_stat_header(int *fctr)
-{
-	printf("Filesystem:           ");
-	if (DISPLAY_KILOBYTES(flags)) {
-		printf("    rkB_nor/s    wkB_nor/s    rkB_dir/s    wkB_dir/s"
-		       "    rkB_svr/s    wkB_svr/s");
-		*fctr = 1024;
-	}
-	else if (DISPLAY_MEGABYTES(flags)) {
-		printf("    rMB_nor/s    wMB_nor/s    rMB_dir/s    wMB_dir/s"
-		       "    rMB_svr/s    wMB_svr/s");
-		*fctr = 1024 * 1024;
-	}
-	else {
-		printf("   rBlk_nor/s   wBlk_nor/s   rBlk_dir/s   wBlk_dir/s"
-		       "   rBlk_svr/s   wBlk_svr/s");
-		*fctr = 512;
-	}
-	printf("     ops/s    rops/s    wops/s\n");
-}
-
 /*
  ***************************************************************************
  * Display extended stats, read from /proc/{diskstats,partitions} or /sys.
@@ -1081,42 +885,6 @@ void write_basic_stat(int curr, unsigned long long itv, int fctr,
 	       (unsigned long long) wr_sec / fctr);
 }
 
-
-/*
- ***************************************************************************
- * Write NFS stats read from /proc/self/mountstats.
- *
- * IN:
- * @curr	Index in array for current sample statistics.
- * @itv		Interval of time.
- * @fctr	Conversion factor.
- * @shi		Structures describing the NFS filesystems.
- * @ioi		Current sample statistics.
- * @ioj		Previous sample statistics.
- ***************************************************************************
- */
-void write_nfs_stat(int curr, unsigned long long itv, int fctr,
-		    struct io_hdr_stats *shi, struct io_nfs_stats *ioni,
-		    struct io_nfs_stats *ionj)
-{
-	if (DISPLAY_HUMAN_READ(flags)) {
-		printf("%-22s\n%23s", shi->name, "");
-	}
-	else {
-		printf("%-22s ", shi->name);
-	}
-	printf("%12.2f %12.2f %12.2f %12.2f %12.2f %12.2f %9.2f %9.2f %9.2f\n",
-	       S_VALUE(ionj->rd_normal_bytes, ioni->rd_normal_bytes, itv) / fctr,
-	       S_VALUE(ionj->wr_normal_bytes, ioni->wr_normal_bytes, itv) / fctr,
-	       S_VALUE(ionj->rd_direct_bytes, ioni->rd_direct_bytes, itv) / fctr,
-	       S_VALUE(ionj->wr_direct_bytes, ioni->wr_direct_bytes, itv) / fctr,
-	       S_VALUE(ionj->rd_server_bytes, ioni->rd_server_bytes, itv) / fctr,
-	       S_VALUE(ionj->wr_server_bytes, ioni->wr_server_bytes, itv) / fctr,
-	       S_VALUE(ionj->rpc_sends, ioni->rpc_sends, itv),
-	       S_VALUE(ionj->nfs_rops,  ioni->nfs_rops,  itv),
-	       S_VALUE(ionj->nfs_wops,  ioni->nfs_wops,  itv));
-}
-
 /*
  ***************************************************************************
  * Print everything now (stats and uptime).
@@ -1145,12 +913,37 @@ void write_stats(int curr, struct tm *rectime)
 			strftime(timestamp, sizeof(timestamp), "%x %X", rectime);
 		}
 		printf("%s\n", timestamp);
+#ifdef DEBUG
+		if (DISPLAY_DEBUG(flags)) {
+			fprintf(stderr, "%s\n", timestamp);
+		}
+#endif
 	}
 
 	/* Interval is multiplied by the number of processors */
 	itv = get_interval(uptime[!curr], uptime[curr]);
 
 	if (DISPLAY_CPU(flags)) {
+#ifdef DEBUG
+		if (DISPLAY_DEBUG(flags)) {
+			/* Debug output */
+			fprintf(stderr, "itv=%llu st_cpu[curr]{ cpu_user=%llu cpu_nice=%llu "
+					"cpu_sys=%llu cpu_idle=%llu cpu_iowait=%llu cpu_steal=%llu "
+					"cpu_hardirq=%llu cpu_softirq=%llu cpu_guest=%llu }\n",
+				itv,
+				st_cpu[curr]->cpu_user,
+				st_cpu[curr]->cpu_nice,
+				st_cpu[curr]->cpu_sys,
+				st_cpu[curr]->cpu_idle,
+				st_cpu[curr]->cpu_iowait,
+				st_cpu[curr]->cpu_steal,
+				st_cpu[curr]->cpu_hardirq,
+				st_cpu[curr]->cpu_softirq,
+				st_cpu[curr]->cpu_guest
+				);
+		}
+#endif
+
 		/* Display CPU utilization */
 		write_cpu_stat(curr, itv);
 	}
@@ -1203,6 +996,33 @@ void write_stats(int curr, struct tm *rectime)
 						/* No activity: Ignore it */
 						continue;
 				}
+#ifdef DEBUG
+				if (DISPLAY_DEBUG(flags)) {
+					/* Debug output */
+					fprintf(stderr, "name=%s itv=%llu fctr=%d ioi{ rd_sectors=%llu "
+							"wr_sectors=%llu rd_ios=%lu rd_merges=%lu rd_ticks=%lu "
+							"wr_ios=%lu wr_merges=%lu wr_ticks=%lu ios_pgr=%lu tot_ticks=%lu "
+							"rq_ticks=%lu dk_drive=%lu dk_drive_rblk=%lu dk_drive_wblk=%lu }\n",
+						shi->name,
+						itv,
+						fctr,
+						ioi->rd_sectors,
+						ioi->wr_sectors,
+						ioi->rd_ios,
+						ioi->rd_merges,
+						ioi->rd_ticks,
+						ioi->wr_ios,
+						ioi->wr_merges,
+						ioi->wr_ticks,
+						ioi->ios_pgr,
+						ioi->tot_ticks,
+						ioi->rq_ticks,
+						ioi->dk_drive,
+						ioi->dk_drive_rblk,
+						ioi->dk_drive_wblk
+						);
+				}
+#endif
 
 				if (DISPLAY_EXTENDED(flags)) {
 					write_ext_stat(curr, itv, fctr, shi, ioi, ioj);
@@ -1214,31 +1034,6 @@ void write_stats(int curr, struct tm *rectime)
 		}
 		printf("\n");
 	}
-
-	if (DISPLAY_NFS(flags)) {
-		struct io_nfs_stats *ioni, *ionj;
-
-		shi = st_hdr_ionfs;
-
-		/* Display NFS stats header */
-		write_nfs_stat_header(&fctr);
-
-		if (!HAS_NFS(flags)) {
-			/* No NFS stats */
-			printf("\n");
-			return;
-		}
-
-		for (i = 0; i < ionfs_nr; i++, shi++) {
-			if (shi->used) {
-	
-				ioni = st_ionfs[curr] + i;
-				ionj = st_ionfs[!curr] + i;
-				write_nfs_stat(curr, itv, fctr, shi, ioni, ionj);
-			}
-		}
-		printf("\n");
-	}
 }
 
 /*
@@ -1300,11 +1095,6 @@ void rw_io_stat_loop(long int count, struct tm *rectime)
 			}
 		}
 
-		/* Read NFS directories stats */
-		if (HAS_NFS(flags)) {
-			read_nfs_stat(curr);
-		}
-
 		/* Get time */
 		get_localtime(rectime);
 
@@ -1379,6 +1169,13 @@ int main(int argc, char **argv)
 			}
 		}
 
+#ifdef DEBUG
+		else if (!strcmp(argv[opt], "--debuginfo")) {
+			flags |= I_D_DEBUG;
+			opt++;
+		}
+#endif
+
 		else if (!strncmp(argv[opt], "-", 1)) {
 			for (i = 1; *(argv[opt] + i); i++) {
 
@@ -1396,11 +1193,6 @@ int main(int argc, char **argv)
 					report_set = TRUE;
 					break;
 
-				case 'h':
-					/* Display an easy-to-read NFS report */
-					flags |= I_D_HUMAN_READ;
-					break;
-	
 				case 'k':
 					if (DISPLAY_MEGABYTES(flags)) {
 						usage(argv[0]);
@@ -1421,12 +1213,6 @@ int main(int argc, char **argv)
 					/* Display device mapper logical name */
 					flags |= I_D_DEVMAP_NAME;
 					break;
-	
-				case 'n':
-					/* Display NFS stats */
-					flags |= I_D_NFS;
-					report_set = TRUE;
-					break;
 
 				case 't':
 					/* Display timestamp */
