@@ -389,17 +389,43 @@ void write_stats_core(int prev, int curr, int dis,
 			/* Check if we want stats about this proc */
 			if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))))
 				continue;
-
+			
 			printf("%-11s %4d", curr_string, cpu - 1);
 
+			/*
+			 * If the CPU is offline then it is omited from /proc/stat
+			 * and the sum of all values is zero.
+			 * (Remember that guest time is already included in user mode.)
+			 */
+			if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
+			     scc->cpu_iowait  + scc->cpu_idle + scc->cpu_steal +
+			     scc->cpu_hardirq + scc->cpu_softirq) == 0) {
+				/*
+				 * Set current struct fields (which have been set to zero)
+				 * to values from previous iteration. Hence their values won't
+				 * jump from zero when the CPU comes back online.
+				 */
+				*scc = *scp;
+
+				printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f"
+				       "  %6.2f  %6.2f  %6.2f\n",
+				       0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
+				continue;
+			}
+
 			/* Recalculate itv for current proc */
 			pc_itv = get_per_cpu_interval(scc, scp);
-
+			
 			if (!pc_itv) {
-				/* Current CPU is offline */
-				printf("    0.00    0.00    0.00    0.00    0.00    0.00"
-				       "    0.00    0.00    0.00\n");
+				/*
+				 * If the CPU is tickless then there is no change in CPU values
+				 * but the sum of values is not zero.
+				 */
+				printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f"
+				       "  %6.2f  %6.2f  %6.2f\n",
+				       0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 100.0);
 			}
+
 			else {
 				printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f"
 				       "  %6.2f  %6.2f  %6.2f\n",
@@ -468,7 +494,7 @@ void write_stats_core(int prev, int curr, int dis,
 
 			if (!pc_itv) {
 				/* Current CPU is offline */
-				printf("    0.00\n");
+				printf(" %9.2f\n", 0.0);
 			}
 			else {
 				printf(" %9.2f\n",
@@ -635,7 +661,7 @@ void read_interrupts_stat(char *file, struct stats_irqcpu *st_ic[], int ic_nr, i
  */
 void rw_mpstat_loop(int dis_hdr, int rows)
 {
-	struct stats_cpu *scc, *scp;
+	struct stats_cpu *scc;
 	int cpu;
 	int curr = 1, dis = 1;
 	unsigned long lines = rows;
@@ -701,12 +727,12 @@ void rw_mpstat_loop(int dis_hdr, int rows)
 		/*
 		 * Resetting the structure not needed since every fields will be set.
 		 * Exceptions are per-CPU structures: Some of them may not be filled
-		 * if corresponding processor is disabled (offline).
+		 * if corresponding processor is disabled (offline). We set them to zero
+		 * to be able to distinguish between offline and tickless CPUs.
 		 */
 		for (cpu = 1; cpu <= cpu_nr; cpu++) {
-			scc = st_cpu[curr]  + cpu;
-			scp = st_cpu[!curr] + cpu;
-			*scc = *scp;
+			scc = st_cpu[curr] + cpu;
+			memset(scc, 0, STATS_CPU_SIZE);
 		}
 
 		/* Get time */
