@@ -73,7 +73,7 @@ __print_funct_t print_cpu_stats(struct activity *a, int prev, int curr,
 		}
 	}
 	
-	for (i = 0; (i < *a->nr) && (i < a->bitmap->b_size + 1); i++) {
+	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
 		
 		/*
 		 * The size of a->buf[...] CPU structure may be different from the default
@@ -85,7 +85,7 @@ __print_funct_t print_cpu_stats(struct activity *a, int prev, int curr,
 		scp = (struct stats_cpu *) ((char *) a->buf[prev] + i * a->msize);
 
 		/*
-		 * Note: *a->nr is in [1, NR_CPUS + 1].
+		 * Note: a->nr is in [1, NR_CPUS + 1].
 		 * Bitmap size is provided for (NR_CPUS + 1) CPUs.
 		 * Anyway, NR_CPUS may vary between the version of sysstat
 		 * used by sadc to create a file, and the version of sysstat
@@ -104,17 +104,53 @@ __print_funct_t print_cpu_stats(struct activity *a, int prev, int curr,
 			}
 			else {
 				printf("     %3d", i - 1);
+
+				/*
+				 * If the CPU is offline then it is omited from /proc/stat:
+				 * All the fields couldn't have been read and the sum of them is zero.
+				 * (Remember that guest time is already included in user mode.)
+				 */
+				if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
+				     scc->cpu_iowait  + scc->cpu_idle + scc->cpu_steal +
+				     scc->cpu_hardirq + scc->cpu_softirq) == 0) {
+					/*
+					 * Set current struct fields (which have been set to zero)
+					 * to values from previous iteration. Hence their values won't
+					 * jump from zero when the CPU comes back online.
+					 */
+					*scc = *scp;
+					
+					printf("    %6.2f    %6.2f    %6.2f"
+					       "    %6.2f    %6.2f    %6.2f",
+					       0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
+					
+					if (DISPLAY_CPU_ALL(a->opt_flags)) {
+						printf("    %6.2f    %6.2f    %6.2f",
+						       0.0, 0.0, 0.0);
+					}
+					printf("\n");
+					continue;
+				}
+				
 				/* Recalculate interval for current proc */
 				g_itv = get_per_cpu_interval(scc, scp);
 				
 				if (!g_itv) {
-					/* Current CPU is offline */
-					printf("      0.00      0.00      0.00"
-					       "      0.00      0.00      0.00");
-					if (DISPLAY_CPU_ALL(a->opt_flags)) {
-						printf("      0.00      0.00      0.00");
+					/*
+					 * If the CPU is tickless then there is no change in CPU values
+					 * but the sum of values is not zero.
+					 */
+					printf("    %6.2f    %6.2f    %6.2f"
+					       "    %6.2f    %6.2f",
+					       0.0, 0.0, 0.0, 0.0, 0.0);
+
+					if (DISPLAY_CPU_DEF(a->opt_flags)) {
+						printf("    %6.2f\n", 100.0);
+					}
+					else if (DISPLAY_CPU_ALL(a->opt_flags)) {
+						printf("    %6.2f    %6.2f    %6.2f    %6.2f\n",
+						       0.0, 0.0, 0.0, 100.0);
 					}
-					printf("\n");
 					continue;
 				}
 			}
@@ -138,7 +174,7 @@ __print_funct_t print_cpu_stats(struct activity *a, int prev, int curr,
 				       ll_sp_value(scp->cpu_user - scp->cpu_guest,
 						   scc->cpu_user - scc->cpu_guest,     g_itv),
 				       ll_sp_value(scp->cpu_nice,    scc->cpu_nice,    g_itv),
-				       ll_sp_value(scp->cpu_sys   ,  scc->cpu_sys   ,  g_itv),
+				       ll_sp_value(scp->cpu_sys,     scc->cpu_sys,     g_itv),
 				       ll_sp_value(scp->cpu_iowait,  scc->cpu_iowait,  g_itv),
 				       ll_sp_value(scp->cpu_steal,   scc->cpu_steal,   g_itv),
 				       ll_sp_value(scp->cpu_hardirq, scc->cpu_hardirq, g_itv),
@@ -200,13 +236,13 @@ __print_funct_t print_irq_stats(struct activity *a, int prev, int curr,
 		printf("\n%-11s      INTR    intr/s\n", timestamp[!curr]);
 	}
 	
-	for (i = 0; (i < *a->nr) && (i < a->bitmap->b_size + 1); i++) {
+	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
 
 		sic = (struct stats_irq *) ((char *) a->buf[curr] + i * a->msize);
 		sip = (struct stats_irq *) ((char *) a->buf[prev] + i * a->msize);
 		
 		/*
-		 * Note: *a->nr is in [0, NR_IRQS + 1].
+		 * Note: a->nr is in [0, NR_IRQS + 1].
 		 * Bitmap size is provided for (NR_IRQS + 1) interrupts.
 		 * Anyway, NR_IRQS may vary between the version of sysstat
 		 * used by sadc to create a file, and the version of sysstat
@@ -719,7 +755,7 @@ __print_funct_t print_serial_stats(struct activity *a, int prev, int curr,
 		       "     brk/s   ovrun/s\n", timestamp[!curr]);
 	}
 
-	for (i = 0; i < *a->nr; i++) {
+	for (i = 0; i < a->nr; i++) {
 
 		ssc = (struct stats_serial *) ((char *) a->buf[curr] + i * a->msize);
 		ssp = (struct stats_serial *) ((char *) a->buf[prev] + i * a->msize);
@@ -770,7 +806,7 @@ __print_funct_t print_disk_stats(struct activity *a, int prev, int curr,
 		       timestamp[!curr]);
 	}
 
-	for (i = 0; i < *a->nr; i++) {
+	for (i = 0; i < a->nr; i++) {
 
 		sdc = (struct stats_disk *) ((char *) a->buf[curr] + i * a->msize);
 
@@ -793,7 +829,7 @@ __print_funct_t print_disk_stats(struct activity *a, int prev, int curr,
 			dev_name = get_devname(sdc->major, sdc->minor,
 					       USE_PRETTY_OPTION(flags));
 		}
-
+		
 		printf("%-11s %9s %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f\n",
 		       timestamp[curr],
 		       /* Confusion possible here between index and minor numbers */
@@ -832,7 +868,7 @@ __print_funct_t print_net_dev_stats(struct activity *a, int prev, int curr,
 		       "   rxcmp/s   txcmp/s  rxmcst/s\n", timestamp[!curr]);
 	}
 
-	for (i = 0; i < *a->nr; i++) {
+	for (i = 0; i < a->nr; i++) {
 
 		sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + i * a->msize);
 
@@ -878,7 +914,7 @@ __print_funct_t print_net_edev_stats(struct activity *a, int prev, int curr,
 		       timestamp[!curr]);
 	}
 
-	for (i = 0; i < *a->nr; i++) {
+	for (i = 0; i < a->nr; i++) {
 
 		snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + i * a->msize);
 
@@ -1623,12 +1659,12 @@ void stub_print_pwr_cpufreq_stats(struct activity *a, int prev, int curr, int di
 	
 	if (!avg_cpufreq) {
 		/* Allocate array of CPU frequency */
-		if ((avg_cpufreq = (unsigned long long *) malloc(sizeof(unsigned long long) * *a->nr))
+		if ((avg_cpufreq = (unsigned long long *) malloc(sizeof(unsigned long long) * a->nr))
 		    == NULL) {
 			perror("malloc");
 			exit(4);
 		}
-		memset(avg_cpufreq, 0, sizeof(unsigned long long) * *a->nr);
+		memset(avg_cpufreq, 0, sizeof(unsigned long long) * a->nr);
 	}
 	
 	if (dis) {
@@ -1636,18 +1672,18 @@ void stub_print_pwr_cpufreq_stats(struct activity *a, int prev, int curr, int di
 		       timestamp[!curr]);
 	}
 
-	for (i = 0; (i < *a->nr) && (i < a->bitmap->b_size + 1); i++) {
+	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
 		
 		/*
 		 * The size of a->buf[...] CPU structure may be different from the default
-		 * sizeof(struct stats_cpu) value if data have been read from a file!
+		 * sizeof(struct stats_pwr_cpufreq) value if data have been read from a file!
 		 * That's why we don't use a syntax like:
 		 * spc = (struct stats_pwr_cpufreq *) a->buf[...] + i;
 		 */
 		spc = (struct stats_pwr_cpufreq *) ((char *) a->buf[curr] + i * a->msize);
 
 		/*
-		 * Note: *a->nr is in [1, NR_CPUS + 1].
+		 * Note: a->nr is in [1, NR_CPUS + 1].
 		 * Bitmap size is provided for (NR_CPUS + 1) CPUs.
 		 * Anyway, NR_CPUS may vary between the version of sysstat
 		 * used by sadc to create a file, and the version of sysstat
