@@ -1,6 +1,6 @@
 /*
  * sar and sadf common routines.
- * (C) 1999-2008 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2009 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -43,6 +43,8 @@
 #define _(string) (string)
 #endif
 
+extern struct act_bitmap cpu_bitmap;
+
 /*
  ***************************************************************************
  * Init a bitmap (CPU, IRQ, etc.).
@@ -75,11 +77,11 @@ void set_bitmap(unsigned char bitmap[], unsigned char value, unsigned int sz)
 void allocate_structures(struct activity *act[])
 {
 	int i, j;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
-		if (*act[i]->nr > 0) {
+		if (act[i]->nr > 0) {
 			for (j = 0; j < 3; j++) {
-				SREALLOC(act[i]->buf[j], void, act[i]->msize * *act[i]->nr);
+				SREALLOC(act[i]->buf[j], void, act[i]->msize * act[i]->nr);
 			}
 		}
 	}
@@ -96,12 +98,13 @@ void allocate_structures(struct activity *act[])
 void free_structures(struct activity *act[])
 {
 	int i, j;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
-		if (*act[i]->nr > 0) {
+		if (act[i]->nr > 0) {
 			for (j = 0; j < 3; j++) {
 				if (act[i]->buf[j]) {
 					free(act[i]->buf[j]);
+					act[i]->buf[j] = NULL;
 				}
 			}
 		}
@@ -345,7 +348,11 @@ void get_itv_value(struct record_header *record_hdr_curr,
 
 /*
  ***************************************************************************
- * Fill rectime structure according to data saved in file header.
+ * Fill the rectime structure with the file's creation date, based on file's
+ * time data saved in file header.
+ * The resulting timestamp is expressed in the locale of the file creator or
+ * in the user's own locale, depending on whether option -t has been used
+ * or not.
  *
  * IN:
  * @flags	Flags for common options and system state.
@@ -355,13 +362,13 @@ void get_itv_value(struct record_header *record_hdr_curr,
  * @rectime	Date and time from file header.
  ***************************************************************************
  */
-void set_hdr_rectime(unsigned int flags, struct tm *rectime,
-		     struct file_header *file_hdr)
+void get_file_timestamp_struct(unsigned int flags, struct tm *rectime,
+			       struct file_header *file_hdr)
 {
 	struct tm *loc_t;
 
 	if (PRINT_TRUE_TIME(flags)) {
-		/* Get local time */
+		/* Get local time. This is just to fill HH:MM:SS fields */
 		get_time(rectime);
 
 		rectime->tm_mday = file_hdr->sa_day;
@@ -400,8 +407,10 @@ void print_report_hdr(unsigned int flags, struct tm *rectime,
 		      struct file_header *file_hdr, int cpu_nr)
 {
 
-	set_hdr_rectime(flags, rectime, file_hdr);
+	/* Get date of file creation */
+	get_file_timestamp_struct(flags, rectime, file_hdr);
 
+	/* Display the header */
 	print_gal_header(rectime, file_hdr->sa_sysname, file_hdr->sa_release,
 			 file_hdr->sa_nodename, file_hdr->sa_machine,
 			 cpu_nr > 1 ? cpu_nr - 1 : 1);
@@ -431,7 +440,7 @@ unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
 
 	sndc = (struct stats_net_dev *) a->buf[curr] + pos;
 
-	while (index < *a->nr) {
+	while (index < a->nr) {
 		sndp = (struct stats_net_dev *) a->buf[ref] + index;
 		if (!strcmp(sndc->interface, sndp->interface)) {
 			/*
@@ -439,13 +448,13 @@ unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
 			 * If a counter has decreased, then we may assume that the
 			 * corresponding interface was unregistered, then registered again.
 			 */
-			if ((sndc->rx_packets        < sndp->rx_packets)        ||
-			    (sndc->tx_packets        < sndp->tx_packets)        ||
-			    (sndc->rx_bytes          < sndp->rx_bytes)          ||
-			    (sndc->tx_bytes          < sndp->tx_bytes)          ||
-			    (sndc->rx_compressed     < sndp->rx_compressed)     ||
-			    (sndc->tx_compressed     < sndp->tx_compressed)     ||
-			    (sndc->multicast         < sndp->multicast)) {
+			if ((sndc->rx_packets    < sndp->rx_packets)    ||
+			    (sndc->tx_packets    < sndp->tx_packets)    ||
+			    (sndc->rx_bytes      < sndp->rx_bytes)      ||
+			    (sndc->tx_bytes      < sndp->tx_bytes)      ||
+			    (sndc->rx_compressed < sndp->rx_compressed) ||
+			    (sndc->tx_compressed < sndp->tx_compressed) ||
+			    (sndc->multicast     < sndp->multicast)) {
 
 				/*
 				 * Special processing for rx_bytes (_packets) and
@@ -501,7 +510,7 @@ unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
 	}
 
 	/* Network interface not found: Look for the first free structure */
-	for (index = 0; index < *a->nr; index++) {
+	for (index = 0; index < a->nr; index++) {
 		sndp = (struct stats_net_dev *) a->buf[ref] + index;
 		if (!strcmp(sndp->interface, "?")) {
 			memset(sndp, 0, STATS_NET_DEV_SIZE);
@@ -509,7 +518,7 @@ unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
 			break;
 		}
 	}
-	if (index >= *a->nr) {
+	if (index >= a->nr) {
 		/* No free structure: Default is structure of same rank */
 		index = pos;
 	}
@@ -546,7 +555,7 @@ unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
 
 	snedc = (struct stats_net_edev *) a->buf[curr] + pos;
 
-	while (index < *a->nr) {
+	while (index < a->nr) {
 		snedp = (struct stats_net_edev *) a->buf[ref] + index;
 		if (!strcmp(snedc->interface, snedp->interface)) {
 			/*
@@ -576,7 +585,7 @@ unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
 	}
 
 	/* Network interface not found: Look for the first free structure */
-	for (index = 0; index < *a->nr; index++) {
+	for (index = 0; index < a->nr; index++) {
 		snedp = (struct stats_net_edev *) a->buf[ref] + index;
 		if (!strcmp(snedp->interface, "?")) {
 			memset(snedp, 0, STATS_NET_EDEV_SIZE);
@@ -584,7 +593,7 @@ unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
 			break;
 		}
 	}
-	if (index >= *a->nr) {
+	if (index >= a->nr) {
 		/* No free structure: Default is structure of same rank */
 		index = pos;
 	}
@@ -619,19 +628,19 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 
 	sdc = (struct stats_disk *) a->buf[curr] + pos;
 
-	while (index < *a->nr) {
+	while (index < a->nr) {
 		sdp = (struct stats_disk *) a->buf[ref] + index;
 		if ((sdc->major == sdp->major) &&
 		    (sdc->minor == sdp->minor)) {
 			/*
 			 * Disk found.
-			 * If a counter has decreased, then we may assume that the
-			 * corresponding device was unregistered, then registered again.
-			 * NB: AFAIK, such a device cannot be unregistered with current
-			 * kernels.
+			 * If all the counters have decreased then the likelyhood
+			 * is that the disk has been unregistered and a new disk inserted.
+			 * If only one or two have decreased then the likelyhood
+			 * is that the counter has simply wrapped.
 			 */
-			if ((sdc->nr_ios < sdp->nr_ios) ||
-			    (sdc->rd_sect < sdp->rd_sect) ||
+			if ((sdc->nr_ios < sdp->nr_ios) &&
+			    (sdc->rd_sect < sdp->rd_sect) &&
 			    (sdc->wr_sect < sdp->wr_sect)) {
 
 				memset(sdp, 0, STATS_DISK_SIZE);
@@ -644,7 +653,7 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 	}
 
 	/* Disk not found: Look for the first free structure */
-	for (index = 0; index < *a->nr; index++) {
+	for (index = 0; index < a->nr; index++) {
 		sdp = (struct stats_disk *) a->buf[ref] + index;
 		if (!(sdp->major + sdp->minor)) {
 			memset(sdp, 0, STATS_DISK_SIZE);
@@ -653,7 +662,7 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 			break;
 		}
 	}
-	if (index >= *a->nr) {
+	if (index >= a->nr) {
 		/* No free structure found: Default is structure of same rank */
 		index = pos;
 	}
@@ -678,11 +687,16 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 void allocate_bitmaps(struct activity *act[])
 {
 	int i;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
-		if (act[i]->bitmap_size) {
-			SREALLOC(act[i]->bitmap, unsigned char,
-				 BITMAP_SIZE(act[i]->bitmap_size));
+		/*
+		 * If current activity has a bitmap which has not already
+		 * been allocated, then allocate it.
+		 * Note that a same bitmap may be used by several activities.
+		 */
+		if (act[i]->bitmap && !act[i]->bitmap->b_array) {
+			SREALLOC(act[i]->bitmap->b_array, unsigned char,
+				 BITMAP_SIZE(act[i]->bitmap->b_size));
 		}
 	}
 }
@@ -698,10 +712,12 @@ void allocate_bitmaps(struct activity *act[])
 void free_bitmaps(struct activity *act[])
 {
 	int i;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
-		if (act[i]->bitmap_size) {
-			free(act[i]->bitmap);
+		if (act[i]->bitmap && act[i]->bitmap->b_array) {
+			free(act[i]->bitmap->b_array);
+			/* Set pointer to NULL to prevent it from being freed again */
+			act[i]->bitmap->b_array = NULL;
 		}
 	}
 }
@@ -723,15 +739,15 @@ void free_bitmaps(struct activity *act[])
 int get_activity_position(struct activity *act[], unsigned int act_flag)
 {
 	int i;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
 		if (act[i]->id == act_flag)
 			break;
 	}
-	
+
 	if (i == NR_ACT)
 		return -1;
-	
+
 	return i;
 }
 
@@ -754,10 +770,10 @@ int get_activity_nr(struct activity *act[], unsigned int option, int count_outpu
 {
 	int i, n = 0;
 	unsigned int msk;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
 		if ((act[i]->options & option) == option) {
-			
+
 			if (HAS_MULTIPLE_OUTPUTS(act[i]->options) && count_outputs) {
 				for (msk = 1; msk < 0x10; msk <<= 1) {
 					if (act[i]->opt_flags & msk) {
@@ -770,7 +786,7 @@ int get_activity_nr(struct activity *act[], unsigned int option, int count_outpu
 			}
 		}
 	}
-	
+
 	return n;
 }
 
@@ -785,7 +801,7 @@ int get_activity_nr(struct activity *act[], unsigned int option, int count_outpu
 void select_all_activities(struct activity *act[])
 {
 	int i;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
 		act[i]->options |= AO_SELECTED;
 	}
@@ -794,6 +810,7 @@ void select_all_activities(struct activity *act[])
 /*
  ***************************************************************************
  * Select CPU activity if no other activities have been explicitly selected.
+ * Also select CPU "all" if no other CPU has been selected.
  *
  * IN:
  * @act	Array of activities.
@@ -805,9 +822,9 @@ void select_all_activities(struct activity *act[])
 void select_default_activity(struct activity *act[])
 {
 	int p;
-	
+
 	p = get_activity_position(act, A_CPU);
-	
+
 	/* Default is CPU activity... */
 	if (!get_activity_nr(act, AO_SELECTED, COUNT_ACTIVITIES)) {
 		/*
@@ -817,10 +834,12 @@ void select_default_activity(struct activity *act[])
 		act[p]->options |= AO_SELECTED;
 	}
 
-	/* If no CPU's have been selected then select CPU "all" */
-	if (IS_SELECTED(act[p]->options) &&
-	    !count_bits(act[p]->bitmap, BITMAP_SIZE(act[p]->bitmap_size))) {
-		act[p]->bitmap[0] |= 0x01;
+	/*
+	 * If no CPU's have been selected then select CPU "all".
+	 * cpu_bitmap bitmap may be used by several activities (A_CPU, A_PWR_CPUFREQ...)
+	 */
+	if (!count_bits(cpu_bitmap.b_array, BITMAP_SIZE(cpu_bitmap.b_size))) {
+		cpu_bitmap.b_array[0] |= 0x01;
 	}
 }
 
@@ -934,21 +953,21 @@ void copy_structures(struct activity *act[], unsigned int id_seq[],
 		     struct record_header record_hdr[], int dest, int src)
 {
 	int i, p;
-	
+
 	memcpy(&record_hdr[dest], &record_hdr[src], RECORD_HEADER_SIZE);
-	
+
 	for (i = 0; i < NR_ACT; i++) {
 
 		if (!id_seq[i])
 			continue;
 
 		if (((p = get_activity_position(act, id_seq[i])) < 0) ||
-		    (*act[p]->nr < 1)) {
+		    (act[p]->nr < 1)) {
 			PANIC(1);
 		}
 
-		memcpy(act[p]->buf[dest], act[p]->buf[src], act[p]->msize * *act[p]->nr);
-		
+		memcpy(act[p]->buf[dest], act[p]->buf[src], act[p]->msize * act[p]->nr);
+
 	}
 }
 
@@ -969,9 +988,9 @@ void read_file_stat_bunch(struct activity *act[], int curr, int ifd, int act_nr,
 {
 	int i, j, p;
 	struct file_activity *fal = file_actlst;
-	
+
 	for (i = 0; i < act_nr; i++, fal++) {
-		
+
 		if ((p = get_activity_position(act, fal->id)) < 0) {
 			/*
 			 * Ignore current activity in file, which is unknown to
@@ -983,17 +1002,17 @@ void read_file_stat_bunch(struct activity *act[], int curr, int ifd, int act_nr,
 				exit(2);
 			}
 		}
-		else if ((*act[p]->nr > 1) && (act[p]->msize > act[p]->fsize)) {
-			for (j = 0; j < *act[p]->nr; j++) {
+		else if ((act[p]->nr > 1) && (act[p]->msize > act[p]->fsize)) {
+			for (j = 0; j < act[p]->nr; j++) {
 				sa_fread(ifd, (char *) act[p]->buf[curr] + j * act[p]->msize,
 					 act[p]->fsize, HARD_SIZE);
 			}
 		}
-		else if (*act[p]->nr > 0) {
-			sa_fread(ifd, act[p]->buf[curr], act[p]->fsize * *act[p]->nr, HARD_SIZE);
+		else if (act[p]->nr > 0) {
+			sa_fread(ifd, act[p]->buf[curr], act[p]->fsize * act[p]->nr, HARD_SIZE);
 		}
 		else {
-			PANIC(*act[p]->nr);
+			PANIC(act[p]->nr);
 		}
 	}
 }
@@ -1054,16 +1073,16 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[],
 
 	/* Read sa data file standard header and allocate activity list */
 	sa_fread(*ifd, file_hdr, FILE_HEADER_SIZE, HARD_SIZE);
-	
+
 	SREALLOC(*file_actlst, struct file_activity, FILE_ACTIVITY_SIZE * file_hdr->sa_nr_act);
 	fal = *file_actlst;
 
 	/* Read activity list */
 	j = 0;
 	for (i = 0; i < file_hdr->sa_nr_act; i++, fal++) {
-		
+
 		sa_fread(*ifd, fal, FILE_ACTIVITY_SIZE, HARD_SIZE);
-		
+
 		if (fal->nr < 1) {
 			/*
 			 * Every activity, known or unknown,
@@ -1075,32 +1094,32 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[],
 		if (fal->id == A_CPU) {
 			a_cpu = TRUE;
 		}
-		
+
 		if ((p = get_activity_position(act, fal->id)) >= 0) {
 			if (fal->size > act[p]->msize) {
 				act[p]->msize = fal->size;
 			}
 			act[p]->fsize = fal->size;
-			*act[p]->nr   = fal->nr;
+			act[p]->nr    = fal->nr;
 			id_seq[j++]   = fal->id;
 		}
 	}
-	
+
 	if (!a_cpu) {
 		/* CPU activity should always be in file */
 		handle_invalid_sa_file(ifd, file_magic, dfile, 0);
 	}
-	
+
 	while (j < NR_ACT) {
 		id_seq[j++] = 0;
 	}
-	
+
 	/* Check that at least one selected activity is available in file */
 	for (i = 0; i < NR_ACT; i++) {
-		
+
 		if (!IS_SELECTED(act[i]->options))
 			continue;
-		
+
 		/* Here is a selected activity: Does it exist in file? */
 		fal = *file_actlst;
 		for (j = 0; j < file_hdr->sa_nr_act; j++, fal++) {
@@ -1154,57 +1173,63 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 
 			p = get_activity_position(act, A_MEMORY);
 			act[p]->opt_flags |= AO_F_MEM_AMT + AO_F_MEM_DIA + AO_F_MEM_SWAP;
-			
+
 			p = get_activity_position(act, A_IRQ);
-			set_bitmap(act[p]->bitmap, ~0, BITMAP_SIZE(act[p]->bitmap_size));
-			
+			set_bitmap(act[p]->bitmap->b_array, ~0,
+				   BITMAP_SIZE(act[p]->bitmap->b_size));
+
 			p = get_activity_position(act, A_CPU);
-			set_bitmap(act[p]->bitmap, ~0, BITMAP_SIZE(act[p]->bitmap_size));
+			set_bitmap(act[p]->bitmap->b_array, ~0,
+				   BITMAP_SIZE(act[p]->bitmap->b_size));
 			act[p]->opt_flags = AO_F_CPU_ALL;
 			break;
-			
+
 		case 'B':
 			SELECT_ACTIVITY(A_PAGE);
 			break;
-			
+
 		case 'b':
 			SELECT_ACTIVITY(A_IO);
 			break;
-			
+
 		case 'C':
 			*flags |= S_F_COMMENT;
 			break;
-			
+
 		case 'd':
 			SELECT_ACTIVITY(A_DISK);
 			break;
-			
+
+		case 'm':
+			SELECT_ACTIVITY(A_PWR_CPUFREQ);
+			break;
+
 		case 'p':
 			*flags |= S_F_DEV_PRETTY;
 			break;
-			
+
 		case 'q':
 			SELECT_ACTIVITY(A_QUEUE);
 			break;
-			
+
 		case 'r':
 			p = get_activity_position(act, A_MEMORY);
 			act[p]->options   |= AO_SELECTED;
 			act[p]->opt_flags |= AO_F_MEM_AMT;
 			break;
-			
+
 		case 'R':
 			p = get_activity_position(act, A_MEMORY);
 			act[p]->options   |= AO_SELECTED;
 			act[p]->opt_flags |= AO_F_MEM_DIA;
 			break;
-			
+
 		case 'S':
 			p = get_activity_position(act, A_MEMORY);
 			act[p]->options   |= AO_SELECTED;
 			act[p]->opt_flags |= AO_F_MEM_SWAP;
 			break;
-			
+
 		case 't':
 			if (caller == C_SAR) {
 				*flags |= S_F_TRUE_TIME;
@@ -1212,39 +1237,40 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 			else
 				return 1;
 			break;
-			
+
 		case 'u':
 			p = get_activity_position(act, A_CPU);
 			act[p]->options |= AO_SELECTED;
 			if (!*(argv[*opt] + i + 1) && argv[*opt + 1] && !strcmp(argv[*opt + 1], K_ALL)) {
 				(*opt)++;
 				act[p]->opt_flags = AO_F_CPU_ALL;
+				return 0;
 			}
 			else {
 				act[p]->opt_flags = AO_F_CPU_DEF;
 			}
-			return 0;
-			
+			break;
+
 		case 'v':
 			SELECT_ACTIVITY(A_KTABLES);
 			break;
-			
+
 		case 'w':
 			SELECT_ACTIVITY(A_PCSW);
 			break;
-			
+
 		case 'W':
 			SELECT_ACTIVITY(A_SWAP);
 			break;
-			
+
 		case 'y':
 			SELECT_ACTIVITY(A_SERIAL);
 			break;
-			
+
 		case 'V':
 			print_version();
 			break;
-			
+
 		default:
 			return 1;
 		}
@@ -1269,58 +1295,86 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
  */
 int parse_sar_n_opt(char *argv[], int *opt, struct activity *act[])
 {
-	if (!strcmp(argv[*opt], K_DEV)) {
-		SELECT_ACTIVITY(A_NET_DEV);
+	char *t;
+
+	for (t = strtok(argv[*opt], ","); t; t = strtok(NULL, ",")) {
+		if (!strcmp(t, K_DEV)) {
+			SELECT_ACTIVITY(A_NET_DEV);
+		}
+		else if (!strcmp(t, K_EDEV)) {
+			SELECT_ACTIVITY(A_NET_EDEV);
+		}
+		else if (!strcmp(t, K_SOCK)) {
+			SELECT_ACTIVITY(A_NET_SOCK);
+		}
+		else if (!strcmp(t, K_NFS)) {
+			SELECT_ACTIVITY(A_NET_NFS);
+		}
+		else if (!strcmp(t, K_NFSD)) {
+			SELECT_ACTIVITY(A_NET_NFSD);
+		}
+		else if (!strcmp(t, K_IP)) {
+			SELECT_ACTIVITY(A_NET_IP);
+		}
+		else if (!strcmp(t, K_EIP)) {
+			SELECT_ACTIVITY(A_NET_EIP);
+		}
+		else if (!strcmp(t, K_ICMP)) {
+			SELECT_ACTIVITY(A_NET_ICMP);
+		}
+		else if (!strcmp(t, K_EICMP)) {
+			SELECT_ACTIVITY(A_NET_EICMP);
+		}
+		else if (!strcmp(t, K_TCP)) {
+			SELECT_ACTIVITY(A_NET_TCP);
+		}
+		else if (!strcmp(t, K_ETCP)) {
+			SELECT_ACTIVITY(A_NET_ETCP);
+		}
+		else if (!strcmp(t, K_UDP)) {
+			SELECT_ACTIVITY(A_NET_UDP);
+		}
+		else if (!strcmp(t, K_SOCK6)) {
+			SELECT_ACTIVITY(A_NET_SOCK6);
+		}
+		else if (!strcmp(t, K_IP6)) {
+			SELECT_ACTIVITY(A_NET_IP6);
+		}
+		else if (!strcmp(t, K_EIP6)) {
+			SELECT_ACTIVITY(A_NET_EIP6);
+		}
+		else if (!strcmp(t, K_ICMP6)) {
+			SELECT_ACTIVITY(A_NET_ICMP6);
+		}
+		else if (!strcmp(t, K_EICMP6)) {
+			SELECT_ACTIVITY(A_NET_EICMP6);
+		}
+		else if (!strcmp(t, K_UDP6)) {
+			SELECT_ACTIVITY(A_NET_UDP6);
+		}
+		else if (!strcmp(t, K_ALL)) {
+			SELECT_ACTIVITY(A_NET_DEV);
+			SELECT_ACTIVITY(A_NET_EDEV);
+			SELECT_ACTIVITY(A_NET_SOCK);
+			SELECT_ACTIVITY(A_NET_NFS);
+			SELECT_ACTIVITY(A_NET_NFSD);
+			SELECT_ACTIVITY(A_NET_IP);
+			SELECT_ACTIVITY(A_NET_EIP);
+			SELECT_ACTIVITY(A_NET_ICMP);
+			SELECT_ACTIVITY(A_NET_EICMP);
+			SELECT_ACTIVITY(A_NET_TCP);
+			SELECT_ACTIVITY(A_NET_ETCP);
+			SELECT_ACTIVITY(A_NET_UDP);
+			SELECT_ACTIVITY(A_NET_SOCK6);
+			SELECT_ACTIVITY(A_NET_IP6);
+			SELECT_ACTIVITY(A_NET_EIP6);
+			SELECT_ACTIVITY(A_NET_ICMP6);
+			SELECT_ACTIVITY(A_NET_EICMP6);
+			SELECT_ACTIVITY(A_NET_UDP6);
+		}
+		else
+			return 1;
 	}
-	else if (!strcmp(argv[*opt], K_EDEV)) {
-		SELECT_ACTIVITY(A_NET_EDEV);
-	}
-	else if (!strcmp(argv[*opt], K_SOCK)) {
-		SELECT_ACTIVITY(A_NET_SOCK);
-	}
-	else if (!strcmp(argv[*opt], K_NFS)) {
-		SELECT_ACTIVITY(A_NET_NFS);
-	}
-	else if (!strcmp(argv[*opt], K_NFSD)) {
-		SELECT_ACTIVITY(A_NET_NFSD);
-	}
-	else if (!strcmp(argv[*opt], K_IP)) {
-		SELECT_ACTIVITY(A_NET_IP);
-	}
-	else if (!strcmp(argv[*opt], K_EIP)) {
-		SELECT_ACTIVITY(A_NET_EIP);
-	}
-	else if (!strcmp(argv[*opt], K_ICMP)) {
-		SELECT_ACTIVITY(A_NET_ICMP);
-	}
-	else if (!strcmp(argv[*opt], K_EICMP)) {
-		SELECT_ACTIVITY(A_NET_EICMP);
-	}
-	else if (!strcmp(argv[*opt], K_TCP)) {
-		SELECT_ACTIVITY(A_NET_TCP);
-	}
-	else if (!strcmp(argv[*opt], K_ETCP)) {
-		SELECT_ACTIVITY(A_NET_ETCP);
-	}
-	else if (!strcmp(argv[*opt], K_UDP)) {
-		SELECT_ACTIVITY(A_NET_UDP);
-	}
-	else if (!strcmp(argv[*opt], K_ALL)) {
-		SELECT_ACTIVITY(A_NET_DEV);
-		SELECT_ACTIVITY(A_NET_EDEV);
-		SELECT_ACTIVITY(A_NET_SOCK);
-		SELECT_ACTIVITY(A_NET_NFS);
-		SELECT_ACTIVITY(A_NET_NFSD);
-		SELECT_ACTIVITY(A_NET_IP);
-		SELECT_ACTIVITY(A_NET_EIP);
-		SELECT_ACTIVITY(A_NET_ICMP);
-		SELECT_ACTIVITY(A_NET_EICMP);
-		SELECT_ACTIVITY(A_NET_TCP);
-		SELECT_ACTIVITY(A_NET_ETCP);
-		SELECT_ACTIVITY(A_NET_UDP);
-	}
-	else
-		return 1;
 
 	(*opt)++;
 	return 0;
@@ -1346,40 +1400,41 @@ int parse_sar_I_opt(char *argv[], int *opt, struct activity *act[])
 {
 	int i, p;
 	unsigned char c;
+	char *t;
 
 	/* Select interrupt activity */
 	p = get_activity_position(act, A_IRQ);
 	act[p]->options |= AO_SELECTED;
-	
-	if (!strcmp(argv[*opt], K_SUM)) {
-		/* Select total number of interrupts */
-		act[p]->bitmap[0] |= 0x01;
-	}
-	else {
-		if (!strcmp(argv[*opt], K_ALL)) {
-			/* Set bit for the first 16 individual interrupts */
-			act[p]->bitmap[0] |= 0xfe;
-			act[p]->bitmap[1] |= 0xff;
-			act[p]->bitmap[2] |= 0x01;
+
+	for (t = strtok(argv[*opt], ","); t; t = strtok(NULL, ",")) {
+		if (!strcmp(t, K_SUM)) {
+			/* Select total number of interrupts */
+			act[p]->bitmap->b_array[0] |= 0x01;
 		}
-		else if (!strcmp(argv[*opt], K_XALL)) {
+		else if (!strcmp(t, K_ALL)) {
+			/* Set bit for the first 16 individual interrupts */
+			act[p]->bitmap->b_array[0] |= 0xfe;
+			act[p]->bitmap->b_array[1] |= 0xff;
+			act[p]->bitmap->b_array[2] |= 0x01;
+		}
+		else if (!strcmp(t, K_XALL)) {
 			/* Set every bit except for total number of interrupts */
-			c = act[p]->bitmap[0];
-			set_bitmap(act[p]->bitmap, ~0, BITMAP_SIZE(act[p]->bitmap_size));
-			act[p]->bitmap[0] = 0xfe | c;
+			c = act[p]->bitmap->b_array[0];
+			set_bitmap(act[p]->bitmap->b_array, ~0,
+				   BITMAP_SIZE(act[p]->bitmap->b_size));
+			act[p]->bitmap->b_array[0] = 0xfe | c;
 		}
 		else {
-			/*
-			 * Get irq number.
-			 */
-			if (strspn(argv[*opt], DIGITS) != strlen(argv[*opt]))
+			/* Get irq number */
+			if (strspn(t, DIGITS) != strlen(t))
 				return 1;
-			i = atoi(argv[*opt]);
-			if ((i < 0) || (i >= act[p]->bitmap_size))
+			i = atoi(t);
+			if ((i < 0) || (i >= act[p]->bitmap->b_size))
 				return 1;
-			act[p]->bitmap[(i + 1) >> 3] |= 1 << ((i + 1) & 0x07);
+			act[p]->bitmap->b_array[(i + 1) >> 3] |= 1 << ((i + 1) & 0x07);
 		}
 	}
+
 	(*opt)++;
 	return 0;
 }
@@ -1404,26 +1459,32 @@ int parse_sar_I_opt(char *argv[], int *opt, struct activity *act[])
 int parse_sa_P_opt(char *argv[], int *opt, unsigned int *flags, struct activity *act[])
 {
 	int i, p;
+	char *t;
 
 	p = get_activity_position(act, A_CPU);
-	
+
 	if (argv[++(*opt)]) {
 		*flags |= S_F_PER_PROC;
-		if (!strcmp(argv[*opt], K_ALL)) {
-			/*
-			 * Set bit for every processor.
-			 * We still don't know if we are going to read stats
-			 * from a file or not...
-			 */
-			set_bitmap(act[p]->bitmap, ~0, BITMAP_SIZE(act[p]->bitmap_size));
-		}
-		else {
-			if (strspn(argv[*opt], DIGITS) != strlen(argv[*opt]))
-				return 1;
-			i = atoi(argv[*opt]);	/* Get cpu number */
-			if ((i < 0) || (i >= act[p]->bitmap_size))
-				return 1;
-			act[p]->bitmap[(i + 1) >> 3] |= 1 << ((i + 1) & 0x07);
+
+		for (t = strtok(argv[*opt], ","); t; t = strtok(NULL, ",")) {
+			if (!strcmp(t, K_ALL)) {
+				/*
+				 * Set bit for every processor.
+				 * We still don't know if we are going to read stats
+				 * from a file or not...
+				 */
+				set_bitmap(act[p]->bitmap->b_array, ~0,
+					   BITMAP_SIZE(act[p]->bitmap->b_size));
+			}
+			else {
+				/* Get cpu number */
+				if (strspn(t, DIGITS) != strlen(t))
+					return 1;
+				i = atoi(t);
+				if ((i < 0) || (i >= act[p]->bitmap->b_size))
+					return 1;
+				act[p]->bitmap->b_array[(i + 1) >> 3] |= 1 << ((i + 1) & 0x07);
+			}
 		}
 		(*opt)++;
 	}
